/**
 * Ext GWT 3.0.0-rc - Ext for GWT
 * Copyright(c) 2007-2011, Sencha, Inc.
 * licensing@sencha.com
 *
 * http://sencha.com/license
 */
package com.sencha.gxt.theme.base.client.frame;

import com.google.gwt.dom.client.NativeEvent;
import com.google.gwt.safecss.shared.SafeStyles;
import com.google.gwt.safecss.shared.SafeStylesUtils;
import com.google.gwt.safehtml.shared.SafeHtml;
import com.google.gwt.safehtml.shared.SafeHtmlBuilder;
import com.sencha.gxt.core.client.dom.XElement;

/**
 * Defines the interface for classes which "frame" a given element.
 * 
 * @see TableFrame
 * @see DivFrame
 */
public interface Frame {

  public static final FrameOptions EMPTY_FRAME = new FrameOptions();
  
  public static class FrameOptions {

    private static SafeStyles EMPTY = SafeStylesUtils.fromTrustedString("");

    private String tabIndex = "";
    private SafeStyles frameStyle;
    private String frameClasses = "";

    public FrameOptions() {

    }

    public FrameOptions(Integer tabIndex, String frameClasses, SafeStyles frameStyle) {
      this.tabIndex = tabIndex == null ? "" : tabIndex.toString();
      this.frameClasses = frameClasses;
      this.frameStyle = frameStyle;
    }

    public String getFrameClasses() {
      return frameClasses;
    }

    public SafeStyles getFrameStyle() {
      if (frameStyle == null) {
        return EMPTY;
      }
      return frameStyle;
    }

    public String getTabIndex() {
      return tabIndex;
    }

    public void setFrameClasses(String frameClasses) {
      this.frameClasses = frameClasses;
    }

    public void setFrameStyle(SafeStyles frameStyle) {
      this.frameStyle = frameStyle;
    }

    public void setTabIndex(String tabIndex) {
      this.tabIndex = tabIndex;
    }

  }

  void onFocus(XElement parent, boolean focus, NativeEvent event);

  void onOver(XElement parent, boolean over, NativeEvent event);

  void onPress(XElement parent, boolean pressed, NativeEvent event);

  void render(SafeHtmlBuilder builder, FrameOptions options, SafeHtml content);

}
